/***********************
dinputfunctions.cpp

File contains implementations of input functions that
initialize direct input,shuts down direct input, etc.
************************/

/***********************
INCLUDES
************************/

#include "dinputfunctions.h"


LPDIRECTINPUT8 di8object;
LPDIRECTINPUTDEVICE8 keyboard;
LPDIRECTINPUTDEVICE8 mouse;
DIKEYSTATE buffer[256];
DIMOUSESTATE mousestate;

/***********************
FUNCTIONS
************************/

/***********************
starts up direct input
************************/

int startupDirectInput(HWND hwnd)
{

 //create main direct input object
 if(FAILED(DirectInput8Create(main_instance, DIRECTINPUT_VERSION, IID_IDirectInput8A, (void**)&di8object, NULL)))
  return -1;

 //create keyboard device
 if(FAILED(di8object->CreateDevice(GUID_SysKeyboard, &keyboard, NULL)))
  return -1;

 //create mouse device
 if(FAILED(di8object->CreateDevice(GUID_SysMouse, &mouse, NULL)))
  return -1;

 //set data format for keyboard
 if(FAILED(keyboard->SetDataFormat(&c_dfDIKeyboard)))
  return -1;

 //set data format for mouse
 if(FAILED(mouse->SetDataFormat(&c_dfDIMouse)))
  return -1;

 //set keyboard behavior
 if(FAILED(keyboard->SetCooperativeLevel(hwnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE)))
  return -1;

 //set mouse behavior
 if(FAILED(mouse->SetCooperativeLevel(hwnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE)))
  return -1;

 //acquire keyboard, cuz we're ready to take input!
 if(keyboard)
  keyboard->Acquire();

 //acquire mouse!
 if(mouse)
  mouse->Acquire();


 return 0;
}



/***********************
shuts down direct input
************************/
void shutdownDirectInput()
{
 if(keyboard)
  keyboard->Unacquire();
 
 if(mouse)
  mouse->Unacquire();

 if(mouse)
  mouse->Release();

 if(keyboard)
  keyboard->Release();

 if(di8object)
  di8object->Release();

}