#include "enemy.h"

Enemy::Enemy()
{
 x = rand()%SCREEN_WIDTH;
 y = rand()%SCREEN_HEIGHT;

 xvelocity = -400.0f/1000;
 yvelocity = (float)(rand()%400)/1000;
}

Enemy::Enemy(int new_x, int new_y)
{
 x = new_x;
 y = new_y;
 xvelocity = 200.0f/1000; //pixels per second
 yvelocity = 150.0f/1000; //pixels per second
 currentframe = ENEMY_STILL;
}

/**************
Gets X coordinate of enemy
***************/

int Enemy::getX()
{
 return x;
}

/**************
Gets Y coordinate of enemy
***************/

int Enemy::getY()
{
 return y;
}

/**************
Gets X velocity of bullet
***************/

float Enemy::getXVelocity()
{
 return xvelocity;
}

/**************
Gets Y velocity of bullet
***************/

float Enemy:: getYVelocity()
{
 return yvelocity;
}

/**************
Gets X coordinate for center of bitmap
***************/

int Enemy::getCenterX()
{
 int centerx = enemyrect.left + (width/2);

 return centerx;
}

/**************
Gets Y coordinate for center of bitmap
***************/

int Enemy::getCenterY()
{
 int centery = enemyrect.top + (height/2);

 return centery;
}

/**************
Gets RECT structure for enemy
***************/
RECT Enemy::getEnemyRect()
{
 return enemyrect;
}

/**************
Gets DDCOLORKEY structure for enemy
***************/

DDCOLORKEY Enemy::getColorKey()
{
 return key;
}


/**************
Sets position of enemy;
***************/
void Enemy::setPos(int new_x, int new_y)
{
 x = new_x;
 y = new_y;
 enemyrect.left = x;
 enemyrect.top = y;
 enemyrect.right = x + width;
 enemyrect.bottom = y + height;
}

void Enemy::setVelocity(float new_x, float new_y)
{
 xvelocity = new_x;
 yvelocity = new_y;
}

/**************
changes velocities if it goes out of bounds
***************/

void Enemy::keepInbounds()
{
 if(x < 0)
 {
  x = 0;
  xvelocity = -xvelocity;
 }

 if(y < 0)
 {
  y = 0;
  yvelocity = -yvelocity;
 }

 if((x + width) > SCREEN_WIDTH)
 {
  x = SCREEN_WIDTH - width;
  xvelocity = -xvelocity;
 }
 

 if((y + height) > SCREEN_HEIGHT)
 {
  y = SCREEN_HEIGHT - height;
  yvelocity = -yvelocity;
 }

}


/**************
draws enemy object at (new_x, new_y)
***************/

void Enemy::drawEnemy(int new_x, int new_y, unsigned int new_currentframe, LPDIRECTDRAWSURFACE7 dest, int scalefactor)
{
 x = new_x;
 y = new_y;
 
 enemyrect.left = x - scalefactor;
 enemyrect.top = y - scalefactor;

 enemyrect.right = (x + width) + scalefactor;
 enemyrect.bottom = (y + height) + scalefactor;

 drawObject(x, y, enemysurface[new_currentframe], dest, scalefactor, key);
}

/**************
loads animation frames for enemy object
***************/

void Enemy::loadFrames()
{
 DDSURFACEDESC2 tempsd;

 ZeroMemory(&tempsd, sizeof(tempsd));
 tempsd.dwSize = sizeof(tempsd);

 enemysurface[0] = newPicture("bitmaps\\ufo.bmp");
 enemysurface[1] = newPicture("bitmaps\\ufo2.bmp");
 enemysurface[2] = newPicture("bitmaps\\enemyexplode.bmp");

 enemyrect.left = x;
 enemyrect.top = y;
  
 enemysurface[0]->GetSurfaceDesc(&tempsd);
 width = (int)tempsd.dwWidth;
 height = (int)tempsd.dwHeight;

 enemyrect.right = enemyrect.left + width;
 enemyrect.bottom = enemyrect.bottom + height;

 key.dwColorSpaceLowValue = RGB16BIT565(0, 0, 0);
 key.dwColorSpaceLowValue = RGB16BIT565(0, 0, 0);

 for(int i = 0; i < 3; i++)
  if(FAILED(enemysurface[i]->SetColorKey(DDCKEY_SRCBLT, &key)))
   if(MessageBox(NULL, "setcolorkey didn't work!", "error!", MB_OK) == IDOK)
    exit(0);
}

/**************
Destructor
***************/
Enemy::~Enemy()
{
 for(int i = 0; i < 3; i++)
  if(enemysurface[i])
  {
   enemysurface[i]->Release();
   enemysurface[i] = NULL;
  }
}