//Jason Alexander
//This is my game and it's called "Blaster One"

#include "globals.h"
#include "resource.h"
#include "ddrawfunctions.h"
#include "ddutil.h"
#include "ship.h"
#include "poweruptypes.h"
#include "bullet.h"
#include "enemy.h"
#include "soundsystem.h"

Ship *ship1 = new Ship(SCREEN_WIDTH/2, SCREEN_HEIGHT-60, SHIP_STILL);
Bullet bullet[MAX_BULLETS];
Enemy *enemy = new Enemy[MAX_ENEMIES];
NAPALM *napalm = new NAPALM();
Powerup *powerup = new Powerup[3];

DWORD starttimer;
DWORD endtimer;
DWORD timerdiff;
HINSTANCE main_instance;
SOUNDEFFECT explode;
SOUNDEFFECT bombexplode;
SOUNDEFFECT gotbomb;
SOUNDEFFECT goalsreached[4];
SONG songs[MAX_SONGS];

/************
FUNCTIONS
*************/


/************
Used to handle windows events
*************/
LRESULT CALLBACK WinProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
 switch(msg)
 {
  case WM_CREATE: //window first created
   {
   //do stuff
    return 0;
   }break;

  case WM_PAINT:
   {

    return 0;
   }break;

  case WM_DESTROY:
   { 
    PostQuitMessage(0);

    return 0;
   }break;

  default: break;
 }//end switch

 return (DefWindowProc(hwnd, msg, wparam, lparam)); //any messages we didn't handle
}//end WinProc

/************
The main function to start the game!
*************/

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hprevinstance, LPSTR lpCmdLine,
                   int nShowCmd)
 

{
 WNDCLASSEX winclass; //generic, basic windows class
 MSG msg;
 HWND hwnd = NULL;
 main_instance = hInstance;
 
 winclass.cbSize = sizeof(WNDCLASSEX);
 winclass.style = CS_VREDRAW | CS_HREDRAW | CS_OWNDC | CS_DBLCLKS;
 winclass.lpfnWndProc = WinProc;  //callback function for handling windows events
 winclass.cbClsExtra = 0;
 winclass.cbWndExtra = 0;
 winclass.hInstance = hInstance; //passed from WinMain
 winclass.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
 winclass.hCursor = LoadCursor(NULL, IDC_ARROW);
 winclass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
 winclass.lpszMenuName = NULL;
 winclass.lpszClassName = "MAINWINCLASS"; //name to refer to this window class
 winclass.hIconSm = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON2)); //small toolbar icon
 
 RegisterClassEx(&winclass); //registers window class

 hwnd = CreateWindowEx(NULL, "MAINWINCLASS", "Blaster One", WS_POPUP | WS_VISIBLE,
                       0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, NULL, NULL, hInstance, NULL);


 if(FAILED(startupGame(hwnd)))
  return -1;

 //MAIN EVENT LOOP
 while(TRUE)
 {

  if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
  {
   if(msg.message == WM_QUIT)
    break; //we're done! break out of infinite loop
   
   TranslateMessage(&msg);
   DispatchMessage(&msg);
  }//end if

  if(FAILED(gameLoop(hwnd)))
   return -1;


 }//end while

 shutdownGame();

 return msg.wParam;
}

/************
Initializes game, loads animations, sets up crap
*************/

int startupGame(HWND hwnd)
{
 ShowCursor(FALSE);  //get rid of mouse cursor
 srand(timeGetTime());  //seeds random number generator

 if(FAILED(startupDirectDraw(SCREEN_WIDTH, SCREEN_HEIGHT,SCREEN_BPP, hwnd)))
  return -1;

 if(FAILED(startupDirectInput(hwnd)))
  return -1;

 if(FAILED(startupSoundSystem()))
  return -1;

 ship1->loadFrames();
 for(int i = 0; i < MAX_ENEMIES; i++)
 {
  enemy[i].loadFrames();
  enemy[i].setPos(rand()%SCREEN_WIDTH, -50);
 }

 napalm->loadFrames(napalmbullet);
 powerup[BOMBSLOT].loadFrames(bombpowerup);
 powerup[EXTRAGUNSLOT].loadFrames(extragunpowerup);
 powerup[RAPIDFIREGUNSLOT].loadFrames(rapidfirepowerup);
 bullet[0].setVelocity(0.0f, -1000.0f/1000);
 bullet[1].setVelocity(-707.0f/1000, -1000.0f/1000);
 bullet[2].setVelocity(707.0f/1000, -1000.0f/1000);


 //loads sound effects and songs
 explode = loadSoundEffect("sounds\\explode.wav", FSOUND_FREE, FSOUND_LOOP_OFF);
 bombexplode = loadSoundEffect("sounds\\bombexplode.wav", FSOUND_FREE, FSOUND_LOOP_OFF);
 gotbomb = loadSoundEffect("sounds\\gotbomb.wav", FSOUND_FREE, FSOUND_LOOP_OFF);
 goalsreached[0] = loadSoundEffect("sounds\\hundredreached.wav", FSOUND_FREE, FSOUND_LOOP_OFF);

 setSoundDefaults(explode, -1, 30, 0);
 setSoundDefaults(bombexplode, -1, 70, -1);
 setSoundDefaults(gotbomb, -1, 70, -1);
 setSoundDefaults(goalsreached[0], -1, 255, -1);

 songs[0] = loadSong("sounds\\heynow.mid");
 songs[1] = loadSong("sounds\\letsroll.mid");
 songs[2] = loadSong("sounds\\synthesizer.mid");
 songs[3] = loadSong("sounds\\triumph.mid");
 songs[4] = loadSong("sounds\\amazoniankiller.mid");
 songs[5] = loadSong("sounds\\runawayherehecomes.mid");
 

 playSong(songs[0], 255);
 drawTitle(backsurface, "bitmaps\\background.bmp");

 return 0;

}

/************
Cleanup function; frees up memory, shuts down DirectX and stuff
*************/

void shutdownGame()
{
 shutdownSoundSystem();
 shutdownDirectDraw();
 shutdownDirectInput();

 delete ship1;
 delete napalm;
 delete [] powerup;
 delete [] enemy;

 for(int i = 0; i < MAX_SONGS; i++)
  freeSong(songs[i]);


 ShowCursor(TRUE);
 
}

/************
Simple test for collision
*************/

bool isCollision(RECT rect1, RECT rect2)
{

 //test each horizontal point on RECT1
 for(int i = rect1.left; i < rect1.right; i++) 
  if(i >= rect2.left && i <= rect2.right) //is it between horizontal boundary?
   //rect1 approaching from bottom
   if(rect1.top >= rect2.top && rect1.top <= rect2.bottom) //is top of rect between vertical boundary of other RECT?
    return true;
   else //rect1 approaching from top
    if(rect1.bottom >= rect2.top && rect1.bottom <= rect2.bottom)
     return true;

 //test each vertical point on RECT1
 for(i = rect1.top; i < rect1.bottom; i++)
  if(i >= rect2.top && i <= rect2.bottom) //is it between vertical boundary?
   //rect1 approaching from right
   if(rect1.left >= rect2.left && rect1.left <= rect2.right)
    return true;
   else //rect1 approaching from left
    if(rect1.right >= rect2.left && rect1.right <= rect2.right)
     return true;

 return false; //no collision happened!
 
}

/************
Main Game Loop
*************/

int gameLoop(HWND hwnd)
{
 static DWORD starttimepassed = timeGetTime(); //used to start timer for powerup appearances
 starttimer = timeGetTime(); //used to start timer each frame for time-based movement
 static DWORD timepassed = 0; //tracks differences between times for powerup
 int closewindow = 0;  //is window closed?
 static int bulletactive = 0;  //is left mouse button pressed and bullets are on the screen?
 static int napalmactive = 0;  //is right mouse button pressed and napalm is on the screen?
 static int maxenemies = 1;  //maximum number of enemies to draw on screen at once.
 static int powerupactive = 0; //is there a  powerup on screen?
 static int hasbomb = 0;  //does player have a bomb?
 static int hasextragun = 0; //does player have extra gun powerup?
 static int hasrapidfiregun = 0;  //does player have rapidfire gun powerup?
 static int bombactive = 0;  //keeps player from holding space and repeatedly setting off bomb
 static DWORD poweruptime = 30; //how many seconds between powerup appearances?
 static int currentpowerup = rand()%3;
 static long score = 0; //keeps track of score
 char textbuffer[20];
 static bool scorereached[4] = {false};

 //read keyboard
 if(FAILED(keyboard->GetDeviceState(sizeof(DIKEYSTATE[256]), (LPVOID)buffer)))
  return -1;

 //read mouse
 if(FAILED(mouse->GetDeviceState(sizeof(DIMOUSESTATE), (LPVOID)&mousestate)))
  return -1;


 //quit out of game!
 if(buffer[DIK_ESCAPE] & 0x80)
 {
  closewindow = 1;
  SendMessage(hwnd, WM_CLOSE, 0, 0);

  return 0;
 }

 if(closewindow)
  return 0;

 //clear surface
 if(FAILED(erase(backsurface, 0, 0, 0)))
  return -1;

 //draw "stars"
 for(int j = 0; j < 15; j++)
  drawPixel(rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT, 255, 255, 255, backsurface, 1);

 //draw score and instructions
 drawText(backsurface, "SCORE:  ", 0, 0);
 drawText(backsurface, ltoa(score, textbuffer, 10), 70, 0);
 drawText(backsurface, "LEFT CLICK TO SHOOT", 0, 20);
 drawText(backsurface, "RIGHT CLICK FOR NAPALM", 0, 40);
 drawText(backsurface, "SPACEBAR for SUPER DUPER BOMB", 0, SCREEN_HEIGHT - 40);
 drawText(backsurface, "(If you have one)", 0, SCREEN_HEIGHT - 20);

 if(hasbomb)
  drawText(backsurface, "OMGWTF YOU HAVE A SUPER DUPER BOMB!!!", SCREEN_WIDTH/2 - 50, 10);



/********************
GAME LOGIC GOES HERE (RENDERING, AI, INPUT AND STUFF)
*********************/
 
 if(mousestate.lX || mousestate.lY)
  ship1->drawShip(ship1->getX() + mousestate.lX, ship1->getY() + mousestate.lY, SHIP_MOVING, backsurface, NULL);
 else
  ship1->drawShip(ship1->getX(), ship1->getY(), SHIP_STILL, backsurface, NULL);

 //draw all enemies
 for(int i = 0; i < maxenemies; i++)
 {
  if(enemy[i].getEnemyRect().right > SCREEN_WIDTH)
   enemy[i].setPos(rand()%SCREEN_WIDTH, enemy[i].getY()); 

  if(enemy[i].getYVelocity() < 300.0f/1000)
   enemy[i].drawEnemy(enemy[i].getX(), enemy[i].getY() + (int)(enemy[i].getYVelocity() * timerdiff), ENEMY_STILL, backsurface, NULL);
  else
   enemy[i].drawEnemy(enemy[i].getX(), enemy[i].getY() + (int)(enemy[i].getYVelocity() * timerdiff), ENEMY_MOVING, backsurface, NULL);
 }

 //keep ship inbounds...allows crossing over
 if(ship1->getShipRect().right < 0)
  ship1->setPos(SCREEN_WIDTH, ship1->getY());

 if(ship1->getX() > SCREEN_WIDTH)
  ship1->setPos(0 - (ship1->getShipRect().right - ship1->getShipRect().left), ship1->getY());
  

 //shoot regular gun
 if(mousestate.rgbButtons[MOUSE_LEFT_BUTTON] & 0x80)
 {
  if(bulletactive != 1)
  {
   for(i = 0; i < MAX_BULLETS; i++)
    bullet[i].setPos((ship1->getShipRect().left + ship1->getShipRect().right)/2, ship1->getY());
  }
  bulletactive = 1;
 }

 //shoot "napalm"
 if(mousestate.rgbButtons[MOUSE_RIGHT_BUTTON] & 0x80)
 {
  if(napalmactive != 1)
   napalm->setPos(ship1->getX(), ship1->getY());
  napalmactive = 1;
 }

 //SPACE button for screen clearing bomb
 if((buffer[DIK_SPACE] & 0x80) && hasbomb)
 {
   playSoundEffect(FSOUND_FREE, bombexplode);
   erase(backsurface, 255, 255, 255);
   
   //find all enemies that are on screen and eliminate them...increase score for each one also
   for(i = 0; i < maxenemies; i++)
   {
    if(enemy[i].getY() > 0 && enemy[i].getY() < SCREEN_HEIGHT)
    {
     score += 1000;
     enemy[i].drawEnemy(enemy[i].getX(), enemy[i].getY(), ENEMY_EXPLODE, backsurface, 30);
    }
     enemy[i].setPos(rand()%SCREEN_WIDTH, -300);
   }

   hasbomb = 0;
 }

 //use number keys 1-6 to change the song that plays
 if(buffer[DIK_1] & 0x80)
   playSong(songs[0], 255);

 if(buffer[DIK_2] & 0x80)
  playSong(songs[1], 255);

 if(buffer[DIK_3] & 0x80)
   playSong(songs[2], 255);


 if(buffer[DIK_4] & 0x80)
   playSong(songs[3], 255);

 if(buffer[DIK_5] & 0x80)
   playSong(songs[4], 255);

 if(buffer[DIK_6] & 0x80)
   playSong(songs[5], 255);

 
 if(hasrapidfiregun)
  {
   bullet[0].setVelocity(0.0f, -3000.0f/1000);
   bullet[1].setVelocity(-2121.0f/1000, -3000.0f/1000);
   bullet[2].setVelocity(2121.0f/1000, -3000.0f/1000);
  }
 else
 {
  bullet[0].setVelocity(0.0f, -1000.0f/1000);
  bullet[1].setVelocity(-707.0f/1000, -1000.0f/1000);
  bullet[2].setVelocity(707.0f/1000, -1000.0f/1000);
 }



 if(bulletactive && hasextragun)
  for(i = 0; i < MAX_BULLETS; i++)
   bullet[i].drawBullet((int)(bullet[i].getX() + bullet[i].getXVelocity() * timerdiff), (int)(bullet[i].getY() + bullet[i].getYVelocity() * timerdiff), backsurface, 3);
 else if(bulletactive)
  bullet[0].drawBullet((int)bullet[0].getX(), (int)(bullet[0].getY() + bullet[0].getYVelocity() * timerdiff), backsurface, 3);

 if(napalmactive)
  napalm->drawPowerUp(napalm->getX(), (int)(napalm->getY() + napalm->getYVelocity() * timerdiff), backsurface, NULL);

 if(powerupactive)
  powerup[currentpowerup].drawPowerUp(powerup[currentpowerup].getX(), (int)(powerup[currentpowerup].getY() + powerup[currentpowerup].getYVelocity() * timerdiff), backsurface, NULL);

 //draw bomb powerup every 3 seconds
 if(timepassed/1000 > 3)
 {
  timepassed = 0;
  starttimepassed = timeGetTime();
  if(powerupactive != 1)
   powerup->setPos(rand()%SCREEN_WIDTH - (powerup->getPowerupRect().right - powerup->getPowerupRect().left), -50);
  powerupactive = 1;
  currentpowerup = rand()%3;
 }
  


  //collison testing for napalm and enemy
 for(i = 0; i < maxenemies; i++)
 if(napalm->getY() > 0 && napalm->getY() < SCREEN_HEIGHT)
  if(isCollision(napalm->getPowerupRect(), enemy[i].getEnemyRect()))
  {
   enemy[i].drawEnemy(enemy[i].getX(), enemy[i].getY(), ENEMY_EXPLODE, backsurface, 30);
   enemy[i].setPos(rand()%SCREEN_WIDTH, -50);
   playSoundEffect(FSOUND_FREE, explode);
   score += 1000;
  }

 //collision testing for bullet and enemy
 if(hasextragun)
 {
 for(i = 0; i < maxenemies; i++)
  for(int j = 0; j < MAX_BULLETS; j++)
   if(isCollision(enemy[i].getEnemyRect(), bullet[j].getBulletRect()))
   {
    bullet[i].setPos(-200, -70);
    enemy[i].drawEnemy(enemy[i].getX(), enemy[i].getY(), ENEMY_EXPLODE, backsurface, 30);
    enemy[i].setPos(rand()%SCREEN_WIDTH, -50);
    playSoundEffect(FSOUND_FREE, explode);
    score += 1000;
   }
 }
 else if(!hasextragun)
 {
  for(i = 0; i < maxenemies; i++)
   if(isCollision(enemy[i].getEnemyRect(), bullet[0].getBulletRect()))
   {
    bullet[0].setPos(-200, -70);
    enemy[i].drawEnemy(enemy[i].getX(), enemy[i].getY(), ENEMY_EXPLODE, backsurface, 30);
    enemy[i].setPos(rand()%SCREEN_WIDTH, -50);
    playSoundEffect(FSOUND_FREE, explode);
    score += 1000;
   }
 }

 //collision testing for ship and enemy
 for(i = 0; i < maxenemies; i++)
  if(isCollision(ship1->getShipRect(), enemy[i].getEnemyRect()))
  {
   drawCircle(backsurface, ship1->getCenterX(), ship1->getCenterY(), 30, 255, 255, 255, NULL);
   score -= 3000;
   hasbomb = 0;
   hasrapidfiregun = 0;
   hasextragun = 0;
  }

 //collision testing for ship and powerup
  if(isCollision(ship1->getShipRect(), powerup[currentpowerup].getPowerupRect()) && powerupactive)
  {
   powerup[currentpowerup].setPos(rand()%SCREEN_WIDTH, -100);
   if(powerupactive)
    playSoundEffect(FSOUND_FREE, gotbomb);
   
   powerupactive = 0;

   if(currentpowerup == BOMBSLOT)
    hasbomb = 1;
   else if(currentpowerup == EXTRAGUNSLOT)
    hasextragun = 1;
   else if (currentpowerup == RAPIDFIREGUNSLOT)
    hasrapidfiregun = 1;
  }

 //reset bulletactive flag when it goes offscreen
 if(hasextragun)
 {
  for(i = 0; i < MAX_BULLETS; i++)
   if(bullet[i].getY() < 0)
   {
    bulletactive = 0;
    bullet[i].setPos(-200, bullet[i].getY());
    break;
   }
 }
 else
 {
  if(bullet[0].getY() < 0 || bullet[0].getY() > SCREEN_HEIGHT)
  {
   bulletactive = 0;
   bullet[0].setPos(-200, bullet[0].getY());
  }
 }

 //reset napalm flag when it goes offscreen
  if(napalm->getY() < 0 || napalm->getY() > SCREEN_HEIGHT)
  {
   napalmactive = 0;
   napalm->setPos(-200, napalm->getY());
  }


  //reset powerup flag when it reaches bottom of screen
  if(powerup[currentpowerup].getY() > SCREEN_HEIGHT)
  {
   powerupactive = 0;
   powerup[currentpowerup].setPos(-200, -70);
  }

 //reset enemy position when it goes offscreen
 for(i = 0; i < maxenemies; i++)
  if(enemy[i].getY() > SCREEN_HEIGHT)
  {
   enemy[i].setPos(rand()%SCREEN_WIDTH, -50);
   score -= 1000;
  }

 if(score <= 8000 && !scorereached[0])
 {
  poweruptime = 30;
  maxenemies = 3;
  scorereached[0] = true;
 }
 if(score > 8000 && score <= 30000 && !scorereached[1])
 {
  poweruptime = 20;
  maxenemies = 8;
  scorereached[1] = true;
 }
 if(score > 30000 && score <= 100000 && !scorereached[2])
 {
  poweruptime = 15;
  maxenemies = 13;
  scorereached[2] = true;
 }
 if(score > 100000 && !scorereached[3])
 {
  poweruptime = 10;
  maxenemies = MAX_ENEMIES;
  scorereached[3] = true;
 }


 if(FAILED(primarysurface->Flip(NULL, DDFLIP_WAIT)))
  return -1;

 endtimer = timeGetTime();
 timepassed = timeGetTime() - starttimepassed;
 timerdiff = endtimer - starttimer;

 return 0;
}
