#include "powerup.h"

/**************
Default constructor
***************/

Powerup::Powerup()
{
 x = rand()%SCREEN_WIDTH;
 y = -50;
 xvelocity = 0;
 yvelocity = 600.0f/1000;
}

/**************
2 parameter constructor
***************/

Powerup::Powerup(int new_x, int new_y)
{
 x = new_x;
 y = new_y;
 xvelocity = 0;
 yvelocity = 200.0f/1000;
}

/**************
4 parameter constructor
***************/

Powerup::Powerup(int new_x, int new_y, float new_xvelocity, float new_yvelocity)
{
 x = new_x;
 y = new_y;
 xvelocity = new_xvelocity;
 yvelocity = new_yvelocity;
}

/**************
gets X coordinate
***************/

int Powerup::getX()
{
 return x;
}

/**************
gets Y coordinate
***************/

int Powerup::getY()
{
 return y;
}

/**************
gets X velocity of powerup
***************/

float Powerup::getXVelocity()
{
 return xvelocity;
}

/**************
gets Y velocity of powerup
***************/
float Powerup::getYVelocity()
{
 return yvelocity;
}

/**************
sets position of power up at (new_x, new_y)
***************/

void Powerup::setPos(int new_x, int new_y)
{
 x = new_x;
 y = new_y;
 poweruprect.left = x;
 poweruprect.top = y;
 poweruprect.right = x + width;
 poweruprect.right = y + height;
}

/**************
sets X velocity of powerup
***************/

void Powerup::setVelocity(float tempx, float tempy)
{
 xvelocity = tempx;
 yvelocity = tempy;
}

/**************
wrapper for drawing power up at (new_x, new_y) to dest surface; scales if necessary
***************/

void Powerup::drawPowerUp(int new_x, int new_y, LPDIRECTDRAWSURFACE7 dest, int scalefactor)
{
 x = new_x;
 y = new_y;
 poweruprect.left = x - scalefactor;
 poweruprect.top = y - scalefactor;
 poweruprect.right = (x + width) + scalefactor;
 poweruprect.bottom = (y + height) + scalefactor;

 drawObject(x, y, powerupsurface, dest, scalefactor, key);

}

/**************
loads frames for power up depending on what type you need to load
***************/
int Powerup::loadFrames(POWERUPTYPE type)
{
 DDSURFACEDESC2 tempsd;  //temporarily holds surface description

 ZeroMemory(&tempsd, sizeof(tempsd));
 tempsd.dwSize = sizeof(tempsd);

 switch(type)
 {
  case bombpowerup : powerupsurface = newPicture("bitmaps\\bombpowerup.bmp"); break;
  case elasticgunpowerup : powerupsurface = newPicture("bitmaps\\elasticpowerup.bmp"); break;
  case extragunpowerup : powerupsurface = newPicture("bitmaps\\extragunpowerup.bmp"); break;
  case rapidfirepowerup : powerupsurface = newPicture("bitmaps\\rapidfirepowerup.bmp"); break;
  case napalmbullet : powerupsurface = newPicture("bitmaps\\napalm.bmp"); break;
 }
  
 //sets top left of rectangle
 poweruprect.left = x;
 poweruprect.top = y;

 if(FAILED(powerupsurface->GetSurfaceDesc(&tempsd)))
  if(MessageBox(NULL, "couldn't get surface description", "error!", MB_OK == IDOK))
   return -1;

 //sets width and height new surface
 width = (int)tempsd.dwWidth;
 height = (int)tempsd.dwHeight;

 //sets bottom right of rectangle
 poweruprect.right = poweruprect.left + width;
 poweruprect.bottom = poweruprect.top + height;

 key.dwColorSpaceLowValue = RGB16BIT565(0, 0, 0);
 key.dwColorSpaceHighValue = RGB16BIT565(0, 0, 0);

 if(FAILED(powerupsurface->SetColorKey(DDCKEY_SRCBLT, &key)))
  if(MessageBox(NULL, "setcolorkey didn't work!", "error!", MB_OK) == IDOK)
   return -1;

 return 0;
}

/**************
gets RECT structure of powerup
***************/

RECT Powerup::getPowerupRect()
{
 return poweruprect;
}

/**************
gets color key of powerup
***************/

DDCOLORKEY Powerup::getColorKey()
{
 return key;
}

/**************
Destructor
***************/

Powerup::~Powerup()
{
 if(powerupsurface)
 {
  powerupsurface->Release();
  powerupsurface = NULL;
 }
}
