/***********************
soundsystem.cpp

File contains wrappers for FMOD sound functions such as
initializing, playing sounds, shutting down sound system, etc.
************************/


/***********************
INCLUDES
************************/
#include "soundsystem.h"

/***********************
FUNCTIONS
************************/

/***********************
initializes fmod sound system
************************/
int startupSoundSystem()
{
 //initializes fmod
 if(FSOUND_Init(MIXRATE, MAX_SOFTWARE_CHANNELS, 0) == FALSE)
  return -1;

 return 0;
}

/***********************
loads song at filename and returns a pointer to a song
************************/

SONG loadSong(LPCSTR filename)
{
 SONG temp;
 temp = FMUSIC_LoadSong(filename);

 return temp;
}

void playSong(SONG temp, int volume)
{
 FMUSIC_SetMasterVolume(temp, volume);
 FMUSIC_StopAllSongs();
 FMUSIC_PlaySong(temp);
}

SOUNDEFFECT loadSoundEffect(LPCSTR filename, int channel, unsigned int flags)
{
 SOUNDEFFECT temp;
 temp = FSOUND_Sample_Load(channel, filename, flags, 0, 0);

 return temp;
}

void playSoundEffect(int channel, SOUNDEFFECT temp)
{
 FSOUND_PlaySound(channel, temp);
}

void setSoundDefaults(SOUNDEFFECT sound, int frequency, int volume, int pan)
{
 FSOUND_Sample_SetDefaults(sound, frequency, volume, pan, -1);
}

void freeSong(SONG temp)
{
 FMUSIC_FreeSong(temp);
}

void shutdownSoundSystem()
{
 FMUSIC_StopAllSongs();

 FSOUND_Close();
}